/******************************************************************************
*******************************************************************************

	Space Impakto DS
	relminator
	http://rel.betterwebber.com


	Cobject class


*******************************************************************************
******************************************************************************/


#include <nds.h>
#include <assert.h>
	
#include "Csprite.h"
#include "Cobject.h"

/******************************************************************************


******************************************************************************/
Cobject::Cobject()
{
	frame =0;
	num_frames = 1;
}

/******************************************************************************


******************************************************************************/
Cobject::Cobject(OamState *_oam,u8 *_gfx, u32 _width, u32 _height, SpriteSize _size, u32 _numframes, E_gfx_location _gfx_location)
{

	oam = _oam;
	anim_counter = 0;
	anim_delay = 3;
	frame = 0;
	num_frames = _numframes;
	if (num_frames <= 0) num_frames = 1;
	
	gfx_location = _gfx_location;
	tile_offset = _width * _height;
		
	if (gfx_location == E_GFX_VRAM)
	{
		
		sprites = new Csprite[num_frames];
		
		assert(sprites);
		
		for (int i = 0; i < num_frames; i++)
		{
			// if not last frame do not increment oam_id
			// else increment so that we change objects automatically
			
			if (i < (num_frames-1))
				sprites[i].create (oam,_width, _height, _size, SpriteColorFormat_256Color,false);
			else	
				sprites[i].create (oam,_width, _height, _size, SpriteColorFormat_256Color,true);

			dmaCopy(_gfx, sprites[i].gfx, tile_offset);
			_gfx += tile_offset;
		}

	}
	else		// ram
	{
		// only allocate 1 instance since we are going to 
		// dma copy every frame
		sprites = new Csprite[1];
		assert(sprites);
		sprites[0].create (oam,_width, _height, _size, SpriteColorFormat_256Color,true);
		dmaCopy(_gfx, sprites[0].gfx, tile_offset);
		gfx_start_offset = _gfx;
	}
	
}

/******************************************************************************


******************************************************************************/
Cobject::~Cobject()
{

	if (sprites)
	{
		delete[] sprites;
	}
}



/******************************************************************************


******************************************************************************/
void Cobject::init(OamState *_oam,u8 *_gfx, u32 _width, u32 _height, SpriteSize _size, u32 _numframes, E_gfx_location _gfx_location)
{

	oam = _oam;
	
	anim_counter = 0;
	anim_delay = 3;
	
	frame = 0;
	num_frames = _numframes;
	if (num_frames <= 0) num_frames = 1;
	
	gfx_location = _gfx_location;
	tile_offset = _width * _height;
		
	if (gfx_location == E_GFX_VRAM)
	{
		
		sprites = new Csprite[num_frames];
		
		assert(sprites);
		
		for (int i = 0; i < num_frames; i++)
		{
			// if not last frame do not increment oam_id
			// else increment so that we change objects automatically
			
			if (i < (num_frames-1))
				sprites[i].create (oam,_width, _height, _size, SpriteColorFormat_256Color,false);
			else	
				sprites[i].create (oam,_width, _height, _size, SpriteColorFormat_256Color,true);

			dmaCopy(_gfx, sprites[i].gfx, tile_offset);
			_gfx += tile_offset;
		}

	}
	else		// ram
	{
		// only allocate 1 instance since we are going to 
		// dma copy every frame
		sprites = new Csprite[1];
		assert(sprites);
		sprites[0].create (oam,_width, _height, _size, SpriteColorFormat_256Color,true);
		dmaCopy(_gfx, sprites[0].gfx, tile_offset);
		gfx_start_offset = _gfx;
	}
	
}

/******************************************************************************


******************************************************************************/
void Cobject::update_oam()
{

	
	// if vram type directly animate
	if (gfx_location==E_GFX_VRAM)
	{
		sprites[frame].update_oam();
	}
	else	// dma copy every fram for ram type
	{
		u8 *offset = gfx_start_offset + (tile_offset* frame);
		dmaCopy(offset, sprites[0].gfx, tile_offset);
		sprites[0].update_oam();
	}
		
}


