/******************************************************************************
*******************************************************************************

	Space Impakto DS
	relminator
	http://rel.betterwebber.com


	Csprite class


*******************************************************************************
******************************************************************************/

 
#include <nds.h>
#include <assert.h>

#include "Csprite.h"


/******************************************************************************


******************************************************************************/
Csprite::Csprite()
{

}

/******************************************************************************


******************************************************************************/
Csprite::~Csprite()
{

	active = false;  
 
	//api: free the graphics
	if(gfx)
	{	
		oamFreeGfx(oam, gfx);
	}

	gfx = 0;
	
}


/******************************************************************************
	Allocates vram mem for gfx
	
	1. if _reset_id_counter = true s_oamID is reset to zero
		as I'm planning to reload all the sprite gfx every level
		including the ships, bullets, etc.
	2. _increase_id is a parameter I added to control animations
		ie. frames of the same object(sprite) share the same oamID so I'll
		only inc the oamID when I create sprites for another object.
		eg. viper[frame0-frame5] has oam_id =0
		enemy[frame0-frame8] has oam_id = 1

******************************************************************************/
void Csprite::create(OamState *_oam,s32 _width, s32 _height, SpriteSize _size, SpriteColorFormat _format, bool _increase_id,bool _reset_id_counter) 
{
	
	static u8 s_oamID = 0;
	if (_reset_id_counter) s_oamID = 0;
	
	x = 0;
	y = 0;
	
	oam = _oam;
	oamID = s_oamID;
	
	active = true;
	width = _width;
	height = _height;
	size = _size;
	format = _format;
    
	//api: allocate a chunk of sprite graphics memory
	gfx = oamAllocateGfx(_oam, size, format);
	
	assert(gfx);
	
	if (_increase_id)
		s_oamID++;
		
}

/******************************************************************************


******************************************************************************/
void Csprite::destroy()
{

	active = false;  
 
	//api: free the graphics
	if(gfx)
	{	
		oamFreeGfx(oam, gfx);
	}

	gfx = 0;

}


/******************************************************************************

	map our sprite to oam entries

******************************************************************************/
void Csprite::update_oam()
{

	//oamSet (OamState *oam, int id, int x, int y, int priority,
	//		  int palette_alpha, SpriteSize size, SpriteColorFormat format,
	//		  const void *gfxOffset, int affineIndex, bool sizeDouble,
	//		  bool hide, bool hflip, bool vflip, bool mosaic)
	oamSet(oam, 
		oamID,				// id 
		x, y,    			// x,y
		0, 					// priority
		0,					// palette alpha
		size,				// size
		format, 			// color format
		gfx, 				// gfx offset
		-1, 				// affine index
		false,				// double if rotating? 
		!active,			// hidden if true
		false,				// hflip
		false, 				// vflip
		false);				// apply mosaic?

}

