/******************************************************************************
*******************************************************************************

	Space Impakto DS
	relminator
	http://rel.betterwebber.com


	Cviper class


*******************************************************************************
******************************************************************************/



#include <nds.h>

#include "Csprite.h"
#include "Cobject.h"
#include "Cshot.h"
#include "Coption.h"
#include "Cenemy.h"
#include "Cviper.h"


/******************************************************************************


******************************************************************************/
Cviper::Cviper()
{
	x = 10;
	y = 96-16;
	frame = 2;
	shot_delay = MAX_VIPER_SHOT_DELAY;
	option_shot_delay = MAX_OPTION_SHOT_DELAY;
}


/******************************************************************************


******************************************************************************/
void Cviper::start_shots(s32 start_x, s32 start_y)
{
	
	for(int i=0; i<MAX_VIPER_BULLETS; i++)
	{
		if (!shots[i].active)
		{
			shots[i].active = true;
			shots[i].x = start_x;
			shots[i].y = start_y;
			shots[i].dx = VIPER_SHOT_SPEED;
			shots[i].dy = 0;
			shots[i].shot_ID = E_SHOT_NORMAL;
			shots[i].frame = (i & 1);
			break;
		}
	}
	
}


/******************************************************************************


******************************************************************************/
void Cviper::start_wave_shots(s32 start_x, s32 start_y)
{

	static s16 sine_off = 0;
	s16 angle;
	
	
	sine_off += 1024;
	for (int j=0; j<2; j++)
	{	
		if (j==0)
			angle = FP_PI+sine_off;
		else
			angle = sine_off;
		        
	
		for (int i=0; i<MAX_VIPER_BULLETS; i++)
		{
			if (!shots[i].active)
			{
				shots[i].active = true;
				shots[i].x = start_x;
				shots[i].y = start_y;
				shots[i].dx = VIPER_SHOT_SPEED-2;
				shots[i].dy = start_y;
				shots[i].angle = angle;
				shots[i].radius = 0;
				shots[i].shot_ID = E_SHOT_WAVE;
				shots[i].frame = 2;
				break;
			}
		}
	}

}

/******************************************************************************


******************************************************************************/
void Cviper::do_shots()
{
	for(int i=0; i<MAX_VIPER_BULLETS; i++)
	{
		if (shots[i].active)
		{
			if (shots[i].shot_ID == E_SHOT_NORMAL)
			{
				shots[i].x += shots[i].dx;
				shots[i].y += shots[i].dy;
			}
			else
			{
				shots[i].angle += 800;
				shots[i].radius += 3000;
				s32 dy = (sinLerp(shots[i].angle)) * (shots[i].radius);
				
				shots[i].x += shots[i].dx;
				shots[i].y = shots[i].dy + (dy >> 24);         
			}
			
			
			if (shots[i].x > SCREEN_WIDTH)
			{
				shots[i].active = false;
			}
			
			
			shots[i].update(); 
			
		}
	}
}

/******************************************************************************


******************************************************************************/
void Cviper::control(Cenemy &target)
{

	dx = 0;
	dy = 0;
	
	int keys = 0;
	scanKeys();

	keys = keysHeld();
	
	direction = E_NEUTRAL;
	
	if (keys & KEY_LEFT)
	{
		direction = E_LEFT;
		if (x > 0) 
			dx -= 3;
	}
	
	if (keys & KEY_RIGHT) 	
	{
		direction = E_RIGHT;
		if (x < SCREEN_WIDTH-sprites[frame].width) 
			dx += 3;
	}
	
	if (keys & KEY_UP)
	{
		direction = E_UP;
		if (y > 0) 
			dy -= 2;
	}
	
	if (keys & KEY_DOWN) 	
	{
		direction = E_DOWN;
		if (y < SCREEN_HEIGHT-sprites[frame].height) 
			dy += 2;
	}
	
	if (keys & KEY_A)
	{

		shot_delay--;
		if(shot_delay == 0)
		{
			
			shot_delay = MAX_VIPER_SHOT_DELAY;
			start_shots(x+24,y+4);
			start_wave_shots(x+24,y+4);
				
		}
		
		option_shot_delay--;
		if(option_shot_delay == 0)
		{
			
			option_shot_delay = MAX_OPTION_SHOT_DELAY;
			for (int i=0; i < MAX_OPTIONS; i++)
			{
				options[i].start_shots(options[i].x+4, options[i].y+4, target.x+16, target.y+8);
			}
			
				
		}
		
	}
	
	

}

/******************************************************************************


******************************************************************************/
void Cviper::update()
{

	static s8 _direction_counter = 0;
	static s8 _direction = 0;
	
	x += dx;
	y += dy;
	
	

 	// check direction
	if (dy < 0)
		_direction = -1;
	else if (dy > 0)
		_direction = 1;
	else
		_direction = 0;
	
	
	// set up dir counter whenever we move up or down
	if (_direction < 0 || direction == E_UP)							// up
	{
		if (_direction_counter < 32) 
			_direction_counter++;
	}
	else if (_direction > 0 || direction == E_DOWN)					// down
	{
		if (_direction_counter > -32) 
			_direction_counter--;
	}
	else		// cool down when not moving up or down
	{
		if (direction == E_NEUTRAL || direction == E_RIGHT || direction == E_LEFT)
		{
			if (_direction_counter > 0) 
				_direction_counter--;
			else if (_direction_counter < 0)
				_direction_counter++;
		}
	}
	
	
	frame = 2+(_direction_counter >> 4);
	
	// copy viper pos to oam compatible sprite struct
	sprites[frame].x = x;
	sprites[frame].y = y; 
	
	
	// move shots and update shots to oam
	do_shots();
	
	// do options and update options to oam
	for (int i=0; i < MAX_OPTIONS; i++)
	{
		options[i].do_shots();
		options[i].update(x+8,y);
	}
	
	// update to oam
	update_oam();
	

	
}

/******************************************************************************


******************************************************************************/
void Cviper::initialize(u8 *viper_gfx, u8 *shot_gfx, u8 *option_gfx, u8 *option_shot_gfx)
{
	
	int width = 32;
	int height = 16;
	SpriteSize size = SpriteSize_32x16; 	
	
	init(&oamMain, viper_gfx, width, height, size, 5, E_GFX_VRAM);
	
	
	//init_bullets
	width = 16;
	height = 8;
	size = SpriteSize_16x8; 
	
	for(int i=0; i<MAX_VIPER_BULLETS; i++)
	{
		shots[i].init(&oamMain, shot_gfx, width, height, size, 4, E_GFX_VRAM);
		shots[i].active = false;
	}
	
	
	for(int i=0; i<MAX_OPTIONS; i++)
	{
		options[i].initialize(option_gfx, option_shot_gfx);
		options[i].angle = (i * FP_PI);
	}
	
}