/*
 *	Space Impakto DS
 *	relminator (Richard Eric M. Lope BSN RN)
 *	http://rel.betterwebber.com
 *
 */
 
 
// #define NDEBUG
 
#include <nds.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <assert.h>


#include "cearn_atan.h"
#include "Cvertexbuffer.h"
#include "Cspline.h"
#include "Csprite.h"
#include "Cobject.h"
#include "Cbullet.h"
#include "Cenemy.h"
#include "Cshot.h"
#include "Coption.h"
#include "Cviper.h"


#include "nds_ship.h"
#include "nds_shot.h"
#include "nds_opt.h"
#include "nds_blt_.h"
#include "nds_en16_.h"
#include "nds_en_64x32_.h"
#include "organ16.h"
#include "gold16.h"



#define  MAX_SPRITES 128
#define  MAX_SHAPES 4

#define  MAX_POINTS 8
#define  MAX_ENEMIES 7
#define  FP_PI 16384 


class Csupershape
{
	public:
	
	void render();
	void update();
	
	s32 x;
	s32 y;
	s32 z;
	
	s32 x_angle;
	s32 y_angle;
	s32 z_angle;
	
	s32 speed;
	u32 wait_counter;
	u32 wait_value;
	
	bool active;
	
	Cvertexbuffer *ps_shape;

};


void Csupershape::render()
{

	glPushMatrix();
	
		glTranslate3f32(x,y,z);
		glRotateXi(x_angle);
		glRotateYi(y_angle);
		glRotateZi(z_angle);
		ps_shape->render(0,0,false);
		//super_shape_vb->render_lines(255,255,255);
		
	glPopMatrix(1);

}


void Csupershape::update()
{
	if (!active)
	{
		wait_counter++;
		if (wait_counter >= wait_value)
		{
		
			active = true;
			wait_counter = 0;
			
		}
	}
	else
	{
		x -= speed;
		if (x < (-6 << 12)) 
		{
			x = 6 << 12;
			active = false;
		}
		x_angle +=128;
	}
}

/******************************************************************************
*******************************************************************************
	MAIN entry point


*******************************************************************************
******************************************************************************/

void init_gl_screen();
void draw_grid();
void init_super_shapes(Csupershape shape[], Cvertexbuffer *supashape[]);
void init_shape_vbs( Cvertexbuffer *supershape_vb[]);



u16 Csprite::num_sprites = 0;



const int GRID_RINGS = 2;
const int GRID_BANDS = 2;
const int GRID_WIDTH = 15;
const int GRID_HEIGHT = 12;

// some constants to make
// our quadric
const int RINGS = 8;
const int BANDS = 8;
const float RADIUS = 1.0f;


Cenemy enemies[MAX_ENEMIES];
Cviper viper;

Cvertexbuffer *grid_vb;
Cvertexbuffer *super_shape_vb[4];

Csupershape shapes[MAX_SHAPES];

int main(void)
{

	
	grid_vb = init_grid (GRID_RINGS, GRID_BANDS, GRID_WIDTH,GRID_HEIGHT,8,8);


	
	init_shape_vbs(super_shape_vb);
	init_super_shapes(shapes, super_shape_vb);
	
	
	videoSetMode(MODE_5_3D);
	videoSetModeSub(MODE_5_2D);
	
	
	
	vramSetBankA(VRAM_A_MAIN_SPRITE);
	vramSetBankB(VRAM_B_MAIN_SPRITE);
	vramSetBankD(VRAM_D_SUB_SPRITE);

	// hmmm... glInit should ba called after consoleDemoinit? why? 
	// maybe because it allocates fonts in the next unmapped VRAM bank?
	init_gl_screen();  	
	
	vramSetBankD(VRAM_D_TEXTURE);
	grid_vb->load_texture((u8*)organ16Bitmap);
	
	//TEMP!!! copy textures (needs a copy texture funk for Cvertexbuffer and decrease text sizes to 64x64)
	super_shape_vb[0]->load_texture((u8*)gold16Bitmap);
	super_shape_vb[1]->i_texture_ID = super_shape_vb[0]->i_texture_ID;
	super_shape_vb[2]->i_texture_ID = grid_vb->i_texture_ID;
	super_shape_vb[3]->i_texture_ID = grid_vb->i_texture_ID;
	
	// console demo init allocates fonts in VRAM?
	// at the next available bank? in this case BANK_C since
	// A B and D are sprite allocated and E is texture allocated?
	consoleDemoInit();
	
	

	iprintf("\x1b[1;1HSpace Impakto DS");
	iprintf("\x1b[2;1HRelminator");
	iprintf("\x1b[4;1HEnemy uses Fixed point");
	iprintf("\x1b[5;1Hcatmull-Rom spline");	
	iprintf("\x1b[6;1Hhttp://rel.betterwebber.com");
	
	iprintf("\x1b[9;1HControls:");
	iprintf("\x1b[10;1HArrows->move");
	iprintf("\x1b[11;1HKey A->Fire weapons");

	
	//api: initialize OAM to 1D mapping with XX byte offsets and no external palette
	oamInit(&oamMain, SpriteMapping_1D_128, false);

	//
	viper.initialize( (u8*)nds_shipTiles, (u8*)nds_shotTiles, (u8*)nds_optTiles, (u8*)nds_blt_Tiles );
	
	
	//Cobject::init(OamState *_oam,u8 *_gfx, u32 _width, u32 _height, SpriteSize _size, u32 _numframes)
	int width = 16;
	int height = 16;
	SpriteSize size = SpriteSize_16x16; 	
	
	enemies[0].init(&oamMain, (u8*)nds_en16_Tiles, width, height, size, 6, E_GFX_VRAM);
	
	// second sprite test
	enemies[1].init(&oamMain, (u8*)nds_en16_Tiles+(16*16*6), width, height, size, 4, E_GFX_VRAM);
	
	for (int i=0; i<enemies[1].num_frames; i++)
	{
		enemies[1].sprites[i].x = 100;
		enemies[1].sprites[i].y = 100;
	}
	
	// test for single frame sprite
	enemies[2].init(&oamMain, (u8*)nds_en16_Tiles+(16*16*10), width, height, size, 1, E_GFX_VRAM);
	
	enemies[2].sprites[0].x = 10;
	enemies[2].sprites[0].y = 100;
	
	// see if oam get inremented after single frame sprite
	enemies[3].init(&oamMain, (u8*)nds_en16_Tiles+(16*16*16), width, height, size, 2, E_GFX_VRAM);
	
	for (int i=0; i< enemies[3].num_frames; i++)
	{
		enemies[3].sprites[i].x = 100;
		enemies[3].sprites[i].y = 10;
	}
	
	
	// see if RAM based enemy would work
	enemies[4].init(&oamMain, (u8*)nds_en16_Tiles+(16*16*10), width, height, size, 6, E_GFX_RAM);
	
	// safe reference
	enemies[4].sprites->x = 200;
	enemies[4].sprites->y = 10;
	
	// see if non-animated RAM based enemy would work
	enemies[5].init(&oamMain, (u8*)nds_en16_Tiles+(16*16*0), width, height, size, 1, E_GFX_RAM);
	
	// alternative but not safe reference just to test
	enemies[5].sprites[0].x = 200;
	enemies[5].sprites[0].y = 100;
	
	
	
	// use a different tilset (64x32)
	width = 64;
	height = 32;
	size = SpriteSize_64x32; 	
	
	enemies[6].init(&oamMain, (u8*)nds_en_64x32_Tiles+(64*32*4), width, height, size, 3, E_GFX_VRAM);
	enemies[6].anim_delay = 7;
	
	for (int i=0; i < enemies[6].num_frames; i++)
	{
		enemies[6].sprites[i].x = 50;
		enemies[6].sprites[i].y = 50;
	}
	
	
	// palette
	dmaCopy(nds_shipPal, SPRITE_PALETTE, 512);


	for (int i=0; i<MAX_ENEMIES; i++)
	{
		enemies[i].init_spline(MAX_POINTS);
	}
	
	
	//enemies[0].init_spline(MAX_POINTS);
	
	u16 total_frame = 0;
	
	
	while(1)
	{ 
		
		
		total_frame++;
		
		
		for (int i=0; i<MAX_ENEMIES; i++)
		{
			enemies[i].update();
			
			enemies[i].update_oam();
		}
		
		viper.control(enemies[6]);
		viper.update();	
		
		glMatrixMode(GL_MODELVIEW);
		
			draw_grid();
			
			for (int i=0; i<MAX_SHAPES; i++)
			{
				shapes[i].update();
				
				if (shapes[i].active)
					shapes[i].render();
			}	
			
		glFlush(0);
		
		// wait retrace
		swiWaitForVBlank();
		
		//api: updates real oam memory 
		oamUpdate(&oamMain);
		
		
		
	/*
	 *
	 *	consoleClear();
	 *	printf("direction= %i  \n",  Gdir);
	 *	printf("direction_counter= %i  \n",  Gdir_c);
	 *	printf("frame offset= %i  \n",  G_frame);
	 *	
	 */
	 			
		
		
		//sangle +=128;
	}

	return 0;
}

void init_gl_screen()
{

	// initialize gl
	glInit();
	
	//enable textures
	glEnable(GL_TEXTURE_2D);
	
	// enable antialiasing
	glEnable(GL_ANTIALIAS);
	
	// setup the rear plane
	glClearColor(0,0,0,31); // BG must be opaque for AA to work
	glClearPolyID(63); // BG must have a unique polygon ID for AA to work
	
	glClearDepth(GL_MAX_DEPTH);

	//this should work the same as the normal gl call
	glViewport(0,0,255,191);
	
	
	//any floating point gl call is being converted to fixed prior to being implemented
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(70, 256.0 / 192.0, 1, 200);
	
	gluLookAt(	0.0, 0.0, 1.0,		//camera possition 
				0.0, 0.0, 0.0,		//look at
				0.0, 1.0, 0.0);		//up

	glMaterialf(GL_AMBIENT, RGB15(31,31,31));
	glMaterialf(GL_DIFFUSE, RGB15(31,31,31));
	glMaterialf(GL_SPECULAR, BIT(15) | RGB15(31,31,31));
	glMaterialf(GL_EMISSION, RGB15(31,31,31));

	//ds uses a table for shinyness..this generates a half-ass one
	glMaterialShinyness();

	//not a real gl function and will likely change
	glPolyFmt(POLY_ALPHA(31) | POLY_CULL_BACK);
}

void draw_grid()
{
	static s32 text_off=0;

		// floor
	glPushMatrix();

		glTranslatef(0.0,-1.0,-1.0);		
		glRotateXi(inttof32(244));
		
		grid_vb->render(text_off,text_off,true);
		
	glPopMatrix(1);
	
	// ceiling
	glPushMatrix();

		glTranslatef(0.0,1.0,-1.0);
		glRotateXi(inttof32(180));
		
		grid_vb->render(text_off,text_off,true);
	
	glPopMatrix(1);
		
	text_off = (text_off+64) & 4095;

}

void init_super_shapes(Csupershape shape[], Cvertexbuffer *supashape[])
{
	
	for (int i=0; i<MAX_SHAPES; i++)
	{
		shape[i].ps_shape = supashape[i];
		
		shape[i].x = 6 << 12;
		shape[i].y = (-1 + qran_range(0, 2)) << 12;
		shape[i].z = (-4 + qran_range(0, 4)) << 12;
	
		shape[i].x_angle = rand();
		shape[i].y_angle = rand();
		shape[i].z_angle = rand();
		
			
		shape[i].speed = qran_range(64, 256);
		
		shape[i].wait_value = (rand() % 15 ) * 60;
		shape[i].wait_counter = 0;
		shape[i].active = false;
		
		
	}

}

void init_shape_vbs( Cvertexbuffer *supershape_vb[])
{

	float a,b,m,n1,n2,n3;
	
	a = 1;
	b = 1;
	m = 4;
	n1 = 0.3;
	n2 = 1.0;
	n3 = 1.0;
	
	
	
	
	supershape_vb[0] = init_super_shape(	
										RINGS,
										BANDS,
										RADIUS,
										2,
										2,
										a,
										b,
										m,
										n1,
										n2,
										n3
											);
	
	a = 5.784;
	b = 1.436;
	m = 8.14;
	n1 = 5.867;
	n2 = -0.630;
	n3 = 15.179;
	
	supershape_vb[1] = init_super_shape(	
										RINGS,
										BANDS,
										RADIUS,
										2,
										2,
										a,
										b,
										m,
										n1,
										n2,
										n3
											);
											
	a = -5.784;
	b = -1.436;
	m = 8.14;
	n1 = 7.867;
	n2 = -0.630;
	n3 = -12.179;
	
	supershape_vb[2] = init_super_shape(	
										RINGS,
										BANDS,
										RADIUS,
										2,
										2,
										a,
										b,
										m,
										n1,
										n2,
										n3
											);

	a = 1;
	b = 3;
	m = 1;
	n1 = 0.3;
	n2 = 1.0;
	n3 = 1.0;
	
	supershape_vb[3] = init_super_shape(	
										RINGS,
										BANDS,
										RADIUS,
										2,
										2,
										a,
										b,
										m,
										n1,
										n2,
										n3
											);
}


